// Zero page variables
// *******************
.const	result	=	$fa
.const	ptr		=	$fb
.const	tmp		=	$fd
.const	old		=	$02

// Special constants
// *****************
.const	special		=	%01101010	// Rule (106)
.const	bgcolor		=	BLACK		// Background color
.const	fgcolor		=	WHITE		// Foreground color
.const	right		=	1			// Right bit
.const	left		=	1			// Left bit
.const	slideshow	=	false		// Change rule (increment)

// Absolute addresses
// ******************
.const	video		=	$2000	// $2000/$6000/$a000/$e000 only!
.const	convtable	=	$8000	// Aligned! 10 bit => 8 bit conversion table
.const	row			=	$8400	// Row buffer
.const	screen		=	$0c00	// Screen page 4

.pc = $0326	

.byte <start,>start
.byte $ed,$f6

start:	
	// Init VIC	
	// ********
	
	ldx #$81
	.if (left==1)	stx row			// Left border => 1
    stx $d40f
    stx $d412
	lda #$3b	
	
	sta $d011	
	sta $d018
	lda #bgcolor	
	sta $d020
	
	// Init screen and random stuff
	// ****************************

initscreen:	
	ldy #$00	
	.if (left==0)	sty row			// Left border => 0
!:
	lda #[[fgcolor*$10]+bgcolor]	// Clear screen
	sta screen+$0000,y
	sta screen+$0100,y
	sta screen+$0200,y
	sta screen+$0300,y
	lda $d41b						// Init "Random" buffer row using SID
	sta row+1,y	
	dey
	bne !-	
		
	// Init tables
	// ***********

createtable:			// This table translate 10 bits to 8 bits using the chosen rule
	sty ptr				// Low byte is 0
	lda #>convtable	
	sta	ptr+1
extloop:	
	lda ptr+1			// Translate single 10 bit word
	sta tmp+1	
	sty tmp
	lda #$80			// Reset result with exit condition
	sta result
inloop:
	lda tmp				// Load triplet
	and #$07
	tax					// Use it as counter
	lsr tmp+1			// Prepare for next triplet
	ror tmp	
specialpointer:
	lda #special		// Compute single bit
!:
	lsr 
	dex
	bpl !-
	ror result			// Store it in the result and check termination bit (this loops 8 times)
	bcc inloop
	lda result			// Store byte
	sta (ptr),y			// in the table
	iny
	bne extloop			// Loop $100
	inc ptr+1
	lda ptr+1
	and #$03
	bne extloop			// Loop $400
	
	// Drawing routine

drawing:		
	ldy #<[8*39]		// Start from top right corner byte
	lda #>[video+8*39]	
	sta ptr+1
	
extmainloop:			// Full screen loop
	ldx #40				// There are 40 bytes in each line
	.if (right==0) clc	// Right bit => 0
			  else sec	// Right bit => 1
	
mainloop:				// Single line loop
	lda row,x			// Load lower 8 bits
	sta convload+1		// and store it on low-address table pointer
	lda row-1,x			// Load higher 8 bits from next byte
	and #$03			// we only need 2 of them
	ora #[>convtable]	// add the table starting position
	sta convload+2		// and use the value as high address table pointer
convload:
	lda convtable		// Load converted value
	rol					// Adjust the value (the 10 bits are misaligned, the result must be rolled) <- Lower bit from previous loop
	sta row,x			// Ok, store the value in the row buffer
	sta (ptr),y			// and in the video memory
	php					// Save carry status (needed for next loop)
	sec					// Move to next byte (on the left)
	tya
	sbc #$08
	tay
	bcs skip
	dec ptr+1
skip:
	plp					// Restore carry
	dex
	bne mainloop		// Loop for the whole row
	ldx #$07			// Update video memory pointer based on 
	axs #$07			// lower 3 bits of it
	bcs !+	
	adc #[<321]
	tay
	lda ptr+1
	adc #[>321]
	sta ptr+1		
	bcc extmainloop
!:	
	adc #<[632]
	tay
	lda ptr+1
	adc #>[632]
	sta ptr+1	
	and #[>video]
	bne extmainloop	
	.if (slideshow) {
			inc specialpointer+1	// Update special pointer
			jmp initscreen
	}
	else	beq drawing	
